(* :Name: CCDPhotometry`Conversions` *)

(* :Title: Various tools for use in the
     reduction of astronomical CCD FITS images *)

(* :Author: Jeff Bryant *)

(* :Context: CCDPhotometry`Conversions` *)

(* :Package Version: 1.0 *)

(* :History:
       Originally written by Jeff Bryant in April 2003.
*)

(* :Keywords:	*)

(* :Source:	*)

(* :Summary:
This package contains various tools for converting FITS image headers
	into more convenient representations that can be operated on.
*)

(* :Mathematica Version: 5.0 *)

(*****************************************************************************)


BeginPackage["CCDPhotometry`Conversions`"]

EarthCoordinates::usage =
"EarthCoordinates[{h, min, sec},{d, m, s}] calculates the x, y, z coordinates of the earth in a heliocentric coordinate system."

SourceCoordinates::usage =
"SourceCoordinates[{h, min, sec},{d, m, s}] calculates the x, y, z coordinates
of the source object in a heliocentric coordinate system, given the geocentric RA and DEC."

HeliocentricRA::usage =  
"HeliocentricRA[modjuldate] calculates the right ascension of the earth in a heliocentric
coordinate system, given the modified Julian date."

HeliocentricDEC::usage =  
"HeliocentricDEC[modjuldate] calculates the declination of the earth in a heliocentric
coordinate system, given the modified Julian date."

Begin["`Private`"]
 
EarthCoordinates[{h_,min_,sec_},{d_,m_,s_}] := 
	Module[{cel, ra, dec},
	  ra = (h + min/60 + sec/3600)*15;
	  dec = (d + m/60 + s/3600);
	  cel = Cos[dec * Pi/180];
    	  {Cos[ra * Pi/180] * cel, Sin[ra * Pi/180] * cel, Sin[dec * Pi/180]}
    ];

SourceCoordinates[{h_,min_,sec_},{d_,m_,s_}] :=
	Module[{cel},
  	  cel = Cos[(d + m/60 + s/3600) * Pi/180.0];
    	  {Cos[(h + min/60 + sec/3600) * 15 * Pi/180] * cel, 
	    Sin[(h + min/60 + sec/3600) * 15 * Pi/180] * cel, 
	    Sin[(d + m/60 + s/3600) * Pi/180]}
    ];
 
HeliocentricRA[modJulDate_] := 
      Module[{eccentricity, eclipticLong, perigeeLong, tropicalYear, 
          obliquity, mjd1980, daysFrom1980, numberOfDeg, numberOfRotations, 
          meanAnomoly, equationOfCenters, solarLongitude, beta, x, y, ra},
	eccentricity = 0.016718;
	eclipticLong = 278.833540;
	perigeeLong = 282.596403;
	tropicalYear = 365.24219572;
	obliquity = 23.441884; 
	mjd1980 = 44238.0; 
	daysFrom1980 = modJulDate - modJulDate * 1980; 
	numberOfDeg = (360.0 * daysFrom1980)/tropicalYear;
	If[(numberOfDeg < 0.0) || (numberOfDeg > 360.0), 
          numberOfRotations = numberOfDeg/360.0;
	  numberOfRotations = Floor[numberOfRotations];
	  numberOfDeg -= numberOfRotations*360.0
	];
	meanAnomoly = numberOfDeg - perigeeLong + eclipticLong;
	equationOfCenters = (360.0 * eccentricity * Sin[meanAnomoly * Pi/180.0])/Pi;
	solarLongitude = numberOfDeg + equationOfCenters + eclipticLong;
	If[solarLongitude > 360.0, solarLongitude -= 360.0];
	beta = 0.0;
	x = Cos[solarLongitude * Pi/180.0];
	y = Sin[solarLongitude * Pi/180.0] * Cos[obliquity * Pi/180.0] - 
          Tan[beta * Pi/180.0] * Sin[obliquity * Pi/180.0];
	ra = ArcTan[y/x];
	ra /= Pi/180.0;
	If[ra < 0.0, ra += 360.0];
	ra /= 15.0;
	ra -= 12.0;
	If[ra < 0.0, ra += 24.0];
	{IntegerPart[ra], IntegerPart[FractionalPart[ra]*60], FractionalPart[FractionalPart[ra]*60]*60}
];

HeliocentricDEC[modJulDate_] := 
      Module[{eccentricity, eclipticLong, perigeeLong, tropicalYear, 
          obliquity, daysFrom1980, numberOfDeg, numberOfRotations, 
          meanAnomoly, equationOfCenters, solarLongitude, beta, dec},
	eccentricity = 0.016718;
	eclipticLong = 278.833540;
	perigeeLong = 282.596403;
	tropicalYear = 365.24219572;
	obliquity = 23.441884; 
	daysFrom1980 = modJulDate - modJulDate * 1980; 
	numberOfDeg = (360.0 * daysFrom1980)/tropicalYear;
	If[(numberOfDeg < 0.0) || (numberOfDeg > 360.0), 
          numberOfRotations = numberOfDeg/360.0;
	  numberOfRotations = Floor[numberOfRotations];
	  numberOfDeg -= numberOfRotations*360.0
	];
	meanAnomoly = numberOfDeg - perigeeLong + eclipticLong;
	equationOfCenters = (360.0 * eccentricity * Sin[meanAnomoly * Pi/180.0])/Pi;
	solarLongitude = numberOfDeg + equationOfCenters + eclipticLong;
	If[solarLongitude > 360.0, solarLongitude -= 360.0];
	beta = 0.0;
	dec = ArcSin[Sin[beta * Pi/180.0] * Cos[obliquity * Pi/180.0] + 
            Cos[beta * Pi/180.0] * Sin[obliquity * Pi/180.0] *
              Sin[solarLongitude * Pi/180.0]];
	dec /= Pi/180.0;
	dec *= -1.0;
	{IntegerPart[dec], IntegerPart[Abs[FractionalPart[dec]]*60], FractionalPart[Abs[FractionalPart[dec]]*60]*60}
];


(*========================================================================*)


 End[ ] (* "CCDPhotometry`Conversions`Private` *)

 EndPackage[ ] (* "CCDPhotometry`Conversions`" *)

