(* :Name: CCDPhotometry`Dates` *)

(* :Title: Various tools for use in the
     reduction of astronomical CCD FITS images *)

(* :Author: Jeff Bryant *)

(* :Context: CCDPhotometry`Dates` *)

(* :Package Version: 1.0 *)

(* :History:
       Originally written by Jeff Bryant in April 2003.
*)

(* :Keywords:	*)

(* :Source:	*)

(* :Summary:
This package contains various tools for computing standard dates from FITS 
	image headers.
*)

(* :Mathematica Version: 5.0 *)

(*****************************************************************************)


BeginPackage["CCDPhotometry`Dates`", "CCDPhotometry`Conversions`"]

JulianDate::usage =
"JulianDate[date_List] returns the Julian date for the specified Gregorian date."

ModifiedJulianDate::usage =
"ModifiedJulianDate[date_List] returns the modified Julian date for the specified 
Gregorian date."

HeliocentricCorrectionTime::usage =  
"HeliocentricCorrectionTime[date_List, ra_List, dec_List] calculates the heliocentric correction time
required to compensate for the light travel time between the earth and the sun."

HeliocentricJulianDate::usage =  
"HeliocentricJulianDate[date_List, ra_List, dec_List] returns the heliocentric Julian date for the 
specified right ascension, declination, and Gregorian date."

Begin["`Private`"]

JulianDate[{y_, m_, d_, h_, mint_, sec_}]:=
	367 y - IntegerPart[7(y + IntegerPart[(m + 9)/12])/4] - 
	IntegerPart[3 (IntegerPart[(y + (m - 9)/7)/100] + 1)/4] + 
	IntegerPart[275 m/9] + d + 1721028.5 + (h + mint/60 + sec/3600)/24;

ModifiedJulianDate[{y_, m_, d_, h_, min_, sec_}] :=
	JulianDate[{y, m, d, h, min, sec}] - 2400000.5;

(* HeliocentricCorrectionTime[date_List, ra_List, dec_List]:=
	Module[{tmpEarth, tmpSource},
		tmpra = CCDPhotometry`Conversions`HeliocentricRA[ModifiedJulianDate[date]];
		tmpdec = CCDPhotometry`Conversions`HeliocentricDEC[ModifiedJulianDate[date]];
		tmpSource = CCDPhotometry`Conversions`SourceCoordinates[tmpra, tmpdec];
		tmpEarth = CCDPhotometry`Conversions`EarthCoordinates[{0,0,0},{0,0,0}];
		499.01265 * (tmpEarth[[1]] * tmpSource[[1]] + tmpEarth[[2]] * tmpSource[[2]] +
		  tmpEarth[[3]] * tmpSource[[3]])
    ]; *)

HeliocentricCorrectionTime[date_List, ra_List, dec_List] :=
	Module[{ALPHA, DELTA, PICON, TT, P, RL, G, AJ, GLM, GLP, TGLP, TGLM, X, Y, A, DEL},
	  ALPHA = ra[[1]] + ra[[2]]/60 + ra[[3]]/3600;
	  DELTA = dec[[1]] + dec[[2]]/60 + dec[[3]]/3600;
	  PICON = 2 Pi/360;
	  TT = (JulianDate[date] - 2400000 - 15020.)/36525.;
	  P = (1.396041 + 0.000308*(TT + 0.5))*(TT - 0.499998);
	  RL = 279.696678 + 36000.76892*TT + 0.000303*TT^2 - P;
	  G = 358.475833 + 35999.04975*TT - 0.00015*TT^2;
	  AJ = 225.444651 + 2880.0*TT + 154.906654*TT;
	  If[G > 360, G -= 360];
	  If[RL > 360, RL -= 360];
	  RL = RL*PICON;
	  G = G*PICON;
	  AJ = AJ*PICON;
	  GLM = G - RL;
	  GLP = G + RL;
	  TGLP = 2.0*G + RL;
	  TGLM = 2.0*G - RL;
	  X = .99986*Cos[RL] - 0.025127*Cos[GLM] + 0.008374*Cos[GLP] + 
	    .000105*Cos[TGLP] + .000063*TT*Cos[GLM] + .000035*Cos[TGLM] - 
	    .000026*Sin[GLM-AJ] - .000021*TT*Cos[GLP];
	  Y = .917308*Sin[RL] + .023053*Sin[GLM] + .007683*Sin[GLP] +
	    .000097*Sin[TGLP] - .000057*TT*Sin[GLM] - .000032*Sin[TGLM] - 
	    .000024*Cos[GLM-AJ] - .000019*TT*Sin[GLP];
	  A = ALPHA*15.0*PICON;
	  DEL = DELTA*PICON;
	  -0.0057755*((Cos[DEL]*Cos[A])*X + (.4337751*Sin[DEL] + 
	    Cos[DEL]*Sin[A])*Y)
	];

HeliocentricJulianDate[date_List, ra_List, dec_List] :=
  JulianDate[date] + HeliocentricCorrectionTime[date, ra, dec];

(*========================================================================*)


 End[ ] (* "CCDPhotometry`Dates`Private` *)

 EndPackage[ ] (* "CCDPhotometry`Dates`" *)

