(* :Name: CCDPhotometry`Headers` *)

(* :Title: Various tools for use in the
     reduction of astronomical CCD FITS images *)

(* :Author: Jeff Bryant *)

(* :Context: CCDPhotometry`Headers` *)

(* :Package Version: 1.0 *)

(* :History:
       Originally written by Jeff Bryant in April 2003.
*)

(* :Keywords:	*)

(* :Source:	*)

(* :Summary:
This package contains various tools for extracting FITS header information.
*)

(* :Mathematica Version: 5.0 *)

(*****************************************************************************)


BeginPackage["CCDPhotometry`Headers`", "CCDPhotometry`ImageLoad`"]

Header::usage = 
"Header[imgobj] returns the full contents of the header data from the specified FITS image object."

HeaderField::usage =
"HeaderField[imgobj_List, field_String] returns the string value of the specified field in the 
specified FITS object."

ImageTime::usage =
"ImageTime[imgobj] returns the starting Universal time of the image acquisition in 
the form {y, m, d, hour, min, sec}."

ImageRA::usage =  
"ImageRA[imgobj] returns the right ascension value contained in the header of the specified
FITS object.  Results of the form {hours, minutes, seconds}."

ImageDEC::usage =  
"ImageRA[imgobj] returns the declination value contained in the header of the specified
FITS object.  Results of the form {degrees, minutes, seconds}."

DateTimeParsingFunctions::usage =  
"DateTimeParsingFunctions is an option for ImageTime that allows the user to specify parsing functions 
for the date and time separately.  The value should be a list of two functions, each one returning a list with 
{year, month, day} and the UT in {hours, minutes, seconds}, respectively."

ParsingFunction::usage =  
"ParsingFunction is an option for both ImageRA and ImageDEC that allows the user to specify a parsing function for extraction 
of the right ascension and declination, respectively.  The value should be a single function that returns the RA or DEC in list format."

Begin["`Private`"]

Header[imgobj_FitsObject] := imgobj[[1, 1, 1]];

HeaderField[imgobj_FitsObject, field_String] := field /. Header[imgobj];
 
ParseDate[imgobj_FitsObject]:=
    Module[{header, datestring, yr, mnth, dy},
      header = Header[imgobj];
      datestring = "Value" /. ("DATE-OBS" /. header);
      {mnth,dy,yr} = 
        ToExpression[
          StringReplace["{" <> StringDrop[StringDrop[datestring, {1}], {-1}] <> "}",
            "/" -> ","]];
      If[50 <= yr <= 99, yr = yr + 1900, yr = yr + 2000];
      {yr, mnth, dy}
    ];

ParseTime[imgobj_FitsObject]:=
    Module[{header, datestring, hr, mnt, scnd},
      header = Header[imgobj];
      timestring = "Value" /. ("UT" /. header);
      {hr, mnt, scnd} = 
        ToExpression[
          StringReplace["{" <> StringDrop[StringDrop[timestring, {1}], {-1}] <> "}",
            ":" -> ","]];
      {hr, mnt, scnd}
    ];

Options[ImageTime] = {DateTimeParsingFunctions -> {ParseDate[#]&, ParseTime[#]&}};

ImageTime::"invalidDate" = "The parsing functions did not return a valid numerical date list.";

ImageTime[imgobj_FitsObject, opts___?OptionQ] := 
    Module[{datefunc, timefunc, outdate},
      {datefunc, timefunc} = DateTimeParsingFunctions /. {opts} /. Options[ImageTime];
      outdate = Join[datefunc@imgobj, timefunc@imgobj];
      If[Length[Cases[Map[NumberQ, outdate], False]] == 0, outdate, Message[ImageTime::"invalidDate"];$Failed]
    ];

RAParsingFunction[imgobj_FitsObject] :=     
    Module[{tmp},
	tmp = Header[imgobj];
	ToExpression[StringReplace[("{" <> StringDrop[
    	  StringDrop[("Value" /. ("RA" /. tmp)), {1}], {-1}] 
	  <> "}"), ":" -> ","]]];

DECParsingFunction[imgobj_FitsObject] :=     
    Module[{tmp},
	tmp = Header[imgobj];
	ToExpression[StringReplace[("{" <> StringDrop[
    	  StringDrop[("Value" /. ("DEC" /. tmp)), {1}], {-1}] 
	  <> "}"), ":" -> ","]]];

Options[ImageRA] = 
   {ParsingFunction -> {RAParsingFunction[#]&}};

ImageRA::"failedParse" = "The parsing function did not return a valid numerical list.";

ImageRA[imgobj_FitsObject, opts___?OptionQ] := 
    Module[{tmp, parsefunc, outra},
	tmp = Header[imgobj];
	{parsefunc} = ParsingFunction /. {opts} /. Options[ImageRA];
	outra = parsefunc@imgobj;
	If[Length[Cases[Map[NumberQ, outra], False]] == 0, outra, Message[ImageRA::"failedParse"];$Failed]
    ];

Options[ImageDEC] = 
   {ParsingFunction -> {DECParsingFunction[#]&}};

ImageDEC::"failedParse" = "The parsing function did not return a valid numerical list.";

ImageDEC[imgobj_FitsObject, opts___?OptionQ] := 
    Module[{tmp, outdec, parsefunc},
	tmp = Header[imgobj];
	{parsefunc} = ParsingFunction /. {opts} /. Options[ImageDEC];
	outdec = parsefunc@imgobj;
	If[Length[Cases[Map[NumberQ, outdec], False]] == 0, outdec, Message[ImageDEC::"failedParse"];$Failed]
    ];	 

(*========================================================================*)


 End[ ] (* "CCDPhotometry`Headers`Private` *)

 EndPackage[ ] (* "CCDPhotometry`Headers`" *)
