(* :Name: CCDPhotometry`Reduction` *)

(* :Title: Various tools for use in the
     reduction of astronomical CCD FITS images *)

(* :Author: Jeff Bryant *)

(* :Context: CCDPhotometry`Reduction` *)

(* :Package Version: 1.0 *)

(* :History:
       Originally written by Jeff Bryant in April 2003.
*)

(* :Keywords:	*)

(* :Source:	*)

(* :Summary:
This package contains a function for the display of FITS image data.
*)

(* :Mathematica Version: 5.0 *)

(*****************************************************************************)


BeginPackage["CCDPhotometry`Reduction`", "CCDPhotometry`ImageLoad`"]

AverageFrame::usage =
"AverageFrame[files] returns an average of FITS objects from a list of specified FITS files. Files can also be FITS image objects."

SubtractFrame::usage = 
"SubtractFrame[file1, file2] subtracts the data of file2 from file1 and returns a new FITS object with
the header of file1. Files can also be FITS image objects."

NormalizeFlat::usage = 
"NormalizeFlat[file] assumes that file is a flat-field image and normalizes the field. Files can also be FITS image objects."

FlatField::usage = 
"FlatField[imagefile, biasfile, normflatfile] assumes that normflatfile is a normalized flat-field and divides 
the bias-subtracted by normflatfile. Files can also be FITS image objects."

RemoveOverscan::usage = 
"RemoveOverscan[file, {lb, ub}] removes the overscan region specified from lowerbound, lb, to upper bound, ub,
 from the FITS data, file. Files can also be FITS image objects."

Begin["`Private`"]

AverageFrame[files_List] := Module[{imgavg, objs},
	objs = Map[ImageLoad[#]&, files];
	imgavg = Transpose[objs[[1]]][[2]];
	FitsObject[{{objs[[1,1,1]], (Plus@@imgavg/Length[files])}}]	
 ];

RemoveOverscan[file_String, {lowbound_Integer, upperbound_Integer}] := Module[{imgobj, procdata, newheader},
	imgobj = ImageLoad[file];
	tmp = imgobj[[1,2]];
	procdata = Transpose[Take[Transpose[tmp], {lowbound, upperbound}]];
	newheader = imgobj[[1,1]] /. ("NAXIS1" -> _) -> ("NAXIS1" -> {"Value" -> upperbound-lowbound+1., "Comment" ->""});
	FitsObject[{{newheader, procdata}}]
 ];

RemoveOverscan[imgobj_FitsObject, {lowbound_Integer, upperbound_Integer}] := 
Module[{procdata, newheader},
	tmp = imgobj[[1, 1, 2]];
	procdata = Transpose[Take[Transpose[tmp], {lowbound, upperbound}]];
	newheader = Header[imgobj] /. ("NAXIS1" -> _) -> ("NAXIS1" -> {"Value" -> upperbound-lowbound+1., "Comment" ->""});
	FitsObject[{{newheader, procdata}}]
 ];

SubtractFrame[file1_String, file2_String] := 
Module[{imgobj1, imgobj2, imgoutdata}, 
	imgobj1 = ImageLoad[file1];
	imgobj2 = ImageLoad[file2];
	imgoutdata = imgobj1[[1,2]] - imgobj2[[1,2]];
	FitsObject[{{imgobj1[[1,1]], imgoutdata}}]
 ];

SubtractFrame[imgobj1_FitsObject, imgobj2_FitsObject] := 
Module[{imgoutdata}, 
	imgoutdata = imgobj1[[1, 1, 2]] - imgobj2[[1, 1, 2]];
	FitsObject[{{Header[imgobj1], imgoutdata}}]
 ];

NormalizeFlat[file_String] := Module[{imgobj, imgoutdata},
	imgobj = ImageLoad[file];
	imgoutdata = imgobj[[1,2]]/((Plus@@Flatten[imgobj[[1,2]]])/Times@@Dimensions[imgobj[[1,2]]]);
	FitsObject[{{imgobj[[1,1]], N[imgoutdata]}}]
 ];

NormalizeFlat[imgobj_FitsObject] := Module[{imgoutdata},
	imgoutdata = imgobj[[1, 1, 2]]/((Plus@@Flatten[imgobj[[1, 1, 2]]])/Times@@Dimensions[imgobj[[1, 1, 2]]]);
	FitsObject[{{Header[imgobj], N[imgoutdata]}}]
 ];

FlatField[file1_String, file2_String, file3_String] := 
Module[{imgobj1, imgobj2, imgobj3, imgoutdata},
	imgobj1 = ImageLoad[file1];
	imgobj2 = ImageLoad[file2];
	imgobj3 = ImageLoad[file3];
	imgoutdata = SubtractFrame[imgobj1, imgobj2][[1, 1, 2]]/imgobj3[[1, 1, 2]];
	FitsObject[{{Header[imgobj1], imgoutdata}}]
];

FlatField[imgobj1_FitsObject, imgobj2_FitsObject, imgobj3_FitsObject] := 
Module[{imgoutdata},
	imgoutdata = SubtractFrame[imgobj1, imgobj2][[1, 1, 2]]/imgobj3[[1, 1, 2]];
	FitsObject[{{Header[imgobj1], imgoutdata}}]
];

(*========================================================================*)


 End[ ] (* "CCDPhotometry`Reduction`Private` *)

 EndPackage[ ] (* "CCDPhotometry`Reduction`" *)
